==> knt_rc_monetized-1.0.0.tar.gz: lib/src/bloc/subscription_bloc.dart <==
import 'package:flutter/services.dart';
import 'package:knt_base_monetized/knt_base_monetized.dart';
import 'package:purchases_flutter/purchases_flutter.dart';

import '../service/iap_service.dart';
import '../util/date_utils.dart';

class SubscriptionBloc extends BaseSubscriptionBloc {
  SubscriptionBloc(this._iapService, MonetizedRepo monetizedRepo)
      : super(monetizedRepo);

  final IapService _iapService;

  @override
  Future<void> connect(bool needSyncFromOldService) async {
    // Get subscriber info from RevenueCat
    final customerInfo = await Purchases.getCustomerInfo();
    final subscriberFirstSeenDate = customerInfo.firstSeen.parse();
    final requestDate = customerInfo.requestDate.parse();
    // User is considered as a new RevenueCat user if the first seen date is
    // not longer than 120 seconds compare to the request date
    final userNewToRevenueCat =
        requestDate.isAfterInRange(120, subscriberFirstSeenDate);
    final hasEntitlement = customerInfo.purchased(_entitlementID);
    if (hasEntitlement) {
      notify(purchased: true);
      return;
    }
    if (!needSyncFromOldService || !userNewToRevenueCat) {
      notify(purchased: false);
      return;
    }

    // Request old system to check available SKUs, then sync to RevenueCat if
    // user has subscription
  }

  @override
  void checkEntitlement() async {
    final customerInfo = await Purchases.getCustomerInfo();
    _updateCustomerInfo(customerInfo);
  }

  @override
  Future<void> purchase(SubscriptionItem item) async {
    final offerings = await Purchases.getOfferings();
    for (final package in offerings.current?.availablePackages ?? <Package>[]) {
      if (package.storeProduct.identifier == item.sku) {
        try {
          final customerInfo = await Purchases.purchasePackage(package);
          _updateCustomerInfo(customerInfo);
        } on PlatformException catch (e) {
          final purchased = PurchasesErrorHelper.getErrorCode(e) ==
              PurchasesErrorCode.productAlreadyPurchasedError;
          notify(purchased: purchased);
        } on Exception {
          // Ignored error caused
          notify(purchased: false);
        }
        return;
      }
    }
  }

  @override
  void restore() async {
    final customerInfo = await Purchases.restorePurchases();
    _updateCustomerInfo(customerInfo);
  }

  @override
  Future<List<SubscriptionItem>> get offerings async {
    final offerings = await Purchases.getOfferings();
    return _iapService
        .packageConverter(offerings.current?.availablePackages ?? []);
  }

  void _updateCustomerInfo(
    CustomerInfo customerInfo, {
    bool isRestoring = false,
  }) {
    final purchased = customerInfo.purchased(_entitlementID);
    notify(purchased: purchased, isRestoring: isRestoring);
  }

  String get _entitlementID => _iapService.entitlementID;
}

extension CustomerInfoExt on CustomerInfo {
  bool purchased(String entitlementID) =>
      entitlements.all[entitlementID]?.isActive ?? false;
}

==> knt_rc_monetized-1.0.1.tar.gz: lib/src/bloc/subscription_bloc.dart <==
import 'package:flutter/services.dart';
import 'package:knt_base_monetized/knt_base_monetized.dart';
import 'package:purchases_flutter/purchases_flutter.dart';

import '../service/iap_service.dart';
import '../util/date_utils.dart';

typedef CustomerInfoHandler = Future<CustomerInfo> Function();

class SubscriptionBloc extends BaseSubscriptionBloc {
  SubscriptionBloc(this._iapService, MonetizedRepo monetizedRepo)
      : super(monetizedRepo);

  final IapService _iapService;

  @override
  Future<void> connect(bool needSyncFromOldService) async {
    // Get subscriber info from RevenueCat
    final customerInfo = await _platformExceptionHandler(
        originHandler: () => Purchases.getCustomerInfo());
    if (customerInfo == null) {
      notify(purchased: false);
      return;
    }

    final subscriberFirstSeenDate = customerInfo.firstSeen.parse();
    final requestDate = customerInfo.requestDate.parse();
    // User is considered as a new RevenueCat user if the first seen date is
    // not longer than 120 seconds compare to the request date
    final userNewToRevenueCat =
        requestDate.isAfterInRange(120, subscriberFirstSeenDate);
    final hasEntitlement = customerInfo.purchased(_entitlementID);
    if (hasEntitlement) {
      notify(purchased: true);
      return;
    }
    if (!needSyncFromOldService || !userNewToRevenueCat) {
      notify(purchased: false);
      return;
    }

    // Request old system to check available SKUs, then sync to RevenueCat if
    // user has subscription
  }

  @override
  void checkEntitlement() async {
    final customerInfo = await _platformExceptionHandler(
        originHandler: () => Purchases.getCustomerInfo());
    _updateCustomerInfo(customerInfo);
  }

  @override
  Future<void> purchase(SubscriptionItem item) async {
    final offerings = await Purchases.getOfferings();
    for (final package in offerings.current?.availablePackages ?? <Package>[]) {
      if (package.storeProduct.identifier == item.sku) {
        final customerInfo = await _platformExceptionHandler(
            originHandler: () => Purchases.purchasePackage(package));
        _updateCustomerInfo(customerInfo);
      }
    }
  }

  @override
  void restore() async {
    final customerInfo = await _platformExceptionHandler(
        originHandler: () => Purchases.restorePurchases());
    _updateCustomerInfo(customerInfo);
  }

  @override
  Future<List<SubscriptionItem>> get offerings async {
    final offerings = await Purchases.getOfferings();
    return _iapService
        .packageConverter(offerings.current?.availablePackages ?? []);
  }

  void _updateCustomerInfo(
    CustomerInfo? customerInfo, {
    bool isRestoring = false,
  }) {
    final purchased = customerInfo?.purchased(_entitlementID) ?? false;
    notify(purchased: purchased, isRestoring: isRestoring);
  }

  Future<CustomerInfo?> _platformExceptionHandler({
    required CustomerInfoHandler originHandler,
  }) async {
    try {
      return await originHandler();
    } on PlatformException catch (e) {
      final purchased = PurchasesErrorHelper.getErrorCode(e) ==
          PurchasesErrorCode.productAlreadyPurchasedError;
      notify(purchased: purchased);
    } on Exception {
      // Ignored error caused
      notify(purchased: false);
    }
    return null;
  }

  String get _entitlementID => _iapService.entitlementID;
}

extension CustomerInfoExt on CustomerInfo {
  bool purchased(String entitlementID) =>
      entitlements.all[entitlementID]?.isActive ?? false;
}
